<?php

class IPay
{
    const PLUGIN_ID = 108;

    /**
     * @var MikroBill
     */
    private $mbApi;
    private $isTestMode;
    private $mch_key;
    private $mch_id;
    private $smch_id;
    private $description;
    private $currency;
    private $trademark;
    private $url_bad;
    private $url_good;
    private $api_url = 'https://api.ipay.ua';
    private $test_api_url = 'https://sandbox-checkout.ipay.ua/api302';

    function __construct($mbApi, $params, $testMode = false)
    {
        $this->mbApi = $mbApi;
        $this->mch_id = $params['mch_id'];
        $this->mch_key = $params['mch_key'];
        $this->gtw_key = $params['gtw_key'];
        $this->smch_id = $params['smch_id'];
        $this->description = $params['description'];
        $this->currency = $params['currency'];
        $this->trademark = $params['trademark'];
        $this->url_good = $params['url_good'];
        $this->url_bad = $params['url_bad'];
        $this->isTestMode = $testMode;
    }

    public function createPayment($login, $amount)
    {
        try {
            $user = $this->mbApi->findUserByLogin($login);
            $description = str_replace('#USER#', $user->contract, $this->description);
            $salt = sha1(microtime(true));;
            $request = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>
        <payment>
            <auth>
                <mch_id>' . $this->mch_id . '</mch_id>
                <salt>' . $salt . '</salt>
                <sign>' . $this->sign($this->mch_key, $salt) . '</sign>
            </auth>
            <urls>
                <good>' . $this->url_good . '</good>
                <bad>' . $this->url_bad . '</bad>
            </urls>
            <transactions>
                <transaction>
                    <amount>' . $amount * 100 . '</amount>
                    <currency>' . $this->currency . '</currency>
                    <desc>' . $description . '</desc>
                    <info>{"shortguid":' . $user->id . '}</info>
                    <smch_id>' . $this->smch_id . '</smch_id>
                </transaction>
            </transactions>
            <trademark>{"ua":"' . $this->trademark . '"}</trademark>
            <lifetime>24</lifetime>
            <lang>ua</lang>
        </payment>';
            $response = $this->send($request);
            $payment = new SimpleXMLElement($response);
            $this->checkSign($payment->sign[0]->__toString(), $payment->salt[0]->__toString(), $this->gtw_key);
            header("Location: $payment->url");
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    private function sign($key, $salt = null)
    {
        if (!$salt) {
            $salt = sha1(microtime(true));
        }
        return hash_hmac('sha512', $salt, $key);
    }

    private function send($request)
    {
        $url = $this->isTestMode ? $this->test_api_url : $this->api_url;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(['data' => $request]));
        $result = curl_exec($ch);
        if (!$result) {
            throw new Exception(curl_error($ch));
        }
        curl_close($ch);
        return $result;
    }

    private function checkSign($sign, $salt, $key)
    {
        if ($sign !== $this->sign($key, $salt)) {
            throw new Exception('Error, wrong sign!');
        }
    }

    /**
     * @param SimpleXMLElement $payment
     */
    public function pay($payment)
    {
        try {
            $this->checkSign($payment->sign->__toString(), $payment->salt->__toString(), $this->gtw_key);
            $paymentStatus = $payment->status->__toString();
            if ($paymentStatus != 5) {
                exit;
            }
            $transaction = $payment->transactions->transaction;
            $transactionId = $transaction->attributes()->id->__toString();
            $invoice = (int)$payment->settlements->settlement->invoice->__toString() / 100;
            $userId = json_decode($transaction->info->__toString(),false, 512, JSON_BIGINT_AS_STRING)->shortguid;
            if ($this->mbApi->isDuplicatePayment($transactionId, 'operations_ipayua')) {
                exit;
            }
            $user = $this->mbApi->findUserById($userId,$invoice);
            $this->mbApi->addMoney($user->id, $GLOBALS ["iPayID"], 'IPay', $invoice);
            $this->mbApi->writeToIpayOperationLog($user->login, $invoice, $transactionId, $paymentStatus);
        } catch (Exception $e) {
            http_response_code(501);
            file_put_contents('payin.txt', sprintf("[%s] %s \n %s \n\n", date('Y-m-d H:i:s'), $e->getMessage(), $payment->asXML()), LOCK_EX|FILE_APPEND);
            echo $e->getMessage();
        }
    }
}