<?php

class Mikrobill
{
    /**
     * @var $db PDO
     * */
    private $db;


    /**
     * @param PDO $connection
     */
    function __construct($connection)
    {
        $this->db = $connection;
        try {
            $this->db->exec("SET NAMES utf8");
            $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
        } catch (PDOException $e) {
            print_r($e->getMessage());
            exit;
        }
    }

    /**
     * @param float $userId Идентификатор пользователя
     * @param string $pluginId Идентификатор плагина плотежной системы в MikroBILL
     * @param string $pluginComment Имя кассира в примечании
     * @param float $amount Сумма пополнения
     */
    public function addMoney($userId, $pluginId, $pluginComment, $amount)
    {
        $pluginId = "VIVAWALLET";// . $pluginId;
        $uid = uniqid("", true) . uniqid("", true);
        $query = "INSERT INTO addcash2 VALUES (:userId, :amount, 0, :pluginId, 0, :uid, :comment , :date)";
        $statement = $this->db->prepare($query);
        $statement->execute([
            ':userId' => $userId,
            ':amount' => $amount,
            ':pluginId' => $pluginId,
            ':uid' => $uid,
            ':comment' => $pluginComment,
            ':date' => date("Y-m-d H:i:s")]);
        $this->writeToActionLog('ADD_MONEY', $userId, $amount, $pluginId);
        $this->refreshDataBase();
    }

    public function writeToActionLog($action, $guid, $amount, $initiator)
    {
        $dateTime = date("Y-m-d H:i:s");
        $statement = $this->db->prepare("INSERT INTO actionslog VALUES (?,?,?,?,'',?);");
        $statement->execute([$dateTime, $action, $guid, $amount, $initiator]);
    }

    public function refreshDataBase()
    {
        $this->db->exec("INSERT INTO `refresh_db` VALUES (1);");
    }

    /**
     * @param $tableName string Имя таблицы с логом платежей
     * @param $id string уникальный идентификатор платежа
     * @param $account
     * @param $pay_amount
     * @return bool
     * @throws Exception
     */
    public function checkPayment($tableName, $id, $account, $pay_amount)
    {
        $statement = $this->db->prepare("SELECT `id` FROM `$tableName` WHERE `operation_id` = ? AND `contract` = ? AND `sum` = ? AND `success` = 1;");
        $statement->execute([$id, $account, $pay_amount]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if ($result !== false) {
            throw new Exception("Payment has already been made");
        }
        $statement = $this->db->prepare("SELECT `id` FROM $tableName WHERE operation_id = ? ;");
        $statement->execute([$id]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if ($result !== false) {
            throw new Exception("A payment with this id has already been made");
        }
        return $result;
    }


    /**
     * @param $login string Логин клиента в личном кабинете
     * @return MikroBillUser
     * @throws Exception
     */
    public function findUser($login)
    {
        $query = 'SELECT shortguid, user_name, pinfo, ballance, user_pay FROM stat LEFT JOIN tarifs USING(tarif_guid) WHERE shortguid = ? OR user_name = ? OR contract = ?';
        $statement = $this->db->prepare($query);
        $statement->execute([$login, $login, $login]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if (!$result) {
            throw new Exception('Пользователь не найден', 10);
        }

        $balance = explode(' ', $result['ballance']);
        $payment = explode(' ', $result['user_pay']);
        $pinfo = explode('||', $result['pinfo']);

        $user = new MikroBillUser();
        $user->id = $result['shortguid'];
        $user->login = $result['user_name'];
        $user->contract = $pinfo[5];
        $user->phone = $pinfo[0];
        $user->email = $pinfo[1];
        $user->address = $pinfo[4];
        $user->balance = (float)$balance[0];
        $user->payment = (float)$payment[0];
        $user->firstName = $pinfo[18];
        $user->lastName = $pinfo[17];
        $user->surName = $pinfo[19];
        $user->fullName = $pinfo[3];
        return $user;
    }


    public function writeToOperationLog($operationLog, $userLogin, $operationId, $amount, $status)
    {
        $query = "INSERT INTO $operationLog (`contract`,`sum`,`operation_id`,`date`, `success`) VALUES (?,?,?,?,?)";
        $statement = $this->db->prepare($query);
        $statement->execute([$userLogin, $amount, $operationId, date("Y-m-d H:i:s"), $status]);
        if ($id = $this->db->lastInsertId()) {
            return $id;
        }
        throw new Exception('Не удалось записать в лог операций');
    }
}

class MikroBillUser
{
    public $id;
    public $login;
    public $contract;
    public $fullName;
    public $address;
    public $phone;
    public $email;
    public $balance;
    public $payment;
    public $firstName;
    public $lastName;
    public $surName;
}