<?php
require_once 'VivaWalletOrder.php';

class VivaWalletAPI
{
    private $merchant;
    private $key;
    private $baseURI;

    public function __construct($uri, $merchant, $key)
    {
        $this->merchant = $merchant;
        $this->key = $key;
        $this->baseURI = rtrim($uri, '/') . '/api';
    }

    /**
     * @param VivaWalletOrder $order
     * @param string $path
     * @return string
     * @throws Exception
     */
    public function createOrder($order, $path = '/orders')
    {
        $response = $this->doRequest($path, $order);
        if ($response->ErrorCode !== 0) {
            throw new Exception($response->ErrorText);
        }
        return number_format($response->OrderCode, 0, '', '');
    }

    public function getTransaction($transactionID)
    {
        $response = $this->doRequest('/transactions/' . $transactionID, null, 'GET');
        if ($response->ErrorCode !== 0) {
            throw new Exception($response->ErrorText);
        }
        return $response->Transactions[0];
    }

    public function getMessagesToken()
    {
        $response = $this->doRequest('/messages/config/token', null, 'GET');
        return $response->Key;
    }

    private function doRequest($path, $params, $method = 'POST')
    {
        $session = curl_init($this->baseURI . $path);
        if ($method === 'POST') {
            $postargs = http_build_query($params);
            curl_setopt($session, CURLOPT_POST, true);
            curl_setopt($session, CURLOPT_POSTFIELDS, $postargs);
        }
        curl_setopt($session, CURLOPT_HEADER, true);
        curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($session, CURLOPT_USERPWD, $this->merchant . ':' . $this->key);
        curl_setopt($session, CURLOPT_SSL_CIPHER_LIST, 'TLSv1');
        $response = curl_exec($session);
        $header_len = curl_getinfo($session, CURLINFO_HEADER_SIZE);
        $resHeader = substr($response, 0, $header_len);
        $resBody = substr($response, $header_len);
        curl_close($session);
        if (!is_object(json_decode($resBody))) {
            preg_match('#^HTTP/1.(?:0|1) [\d]{3} (.*)$#m', $resHeader, $match);
			
            throw new Exception("API Call failed! The error was: " . trim($match[1]));
        }
        return json_decode($resBody);
    }

}